﻿; MoveResizeWindow.ahk[UTF-8] (2024/05/13 16:00)
; for AutoHotKey v2.0.12.0 or later
;
; 指定のウィンドウを移動やリサイズする(最小化・最大化状態ではリサイズが出来ないのでWinRestoreする)
;    MoveResizeWindow PosSize Param (Activate) (NoDispError) (Debug)
;       PosSize
;          X, Y, Width, Height を両端""で囲って指定
;       Param
;          ウィンドウ文字列/ahk_exe フルパス/ahk_class クラスID など両端""でAutoHotKey
;          独自のウィンドウ指定記述、現在アクティブなウィンドウを指定する場合は"A"
;          https://ahkscript.github.io/ja/docs/v2/misc/WinTitle.htm
;          対象ウィンドウのウィンドウスタイルは WS_VISIBLE ビットが有効で WS_DISABLED ビットが無効の
;          ものとしています。複数ウィンドウに該当する場合はエラー表示し処理しません。
;          ex) "ahk_exe C:\Program Files\Mozilla Firefox\firefox.exe"
;          ex) "ahk_exe C:\Program Files\Vivaldi\Application\vivaldi.exe ahk_class Chrome_WidgetWin_1"
;       Activate
;          1/on/activate : ウィンドウをアクティブにする(別の仮想デスクトップでは動作しない)
;          [any] : 上記以外が指定されるか無指定ならアクティブにはしない
;       NoDispError
;          1/on/nodisp : 対象ウィンドウが見つからない時や複数ヒットした際のエラーダイアログを出さない
;          [any] : 上記以外が指定されるか無指定ならエラー時にダイアログが出る
;       Debug
;          [any] : MsgBoxでクラス名とウィンドウスタイルを表示、無指定なら表示なし
;

/* 同時に複数実行を許可する */
#SingleInstance Off

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておかないと仮想デスクトップで動作しない */
DetectHiddenWindows True

WS_POPUP        := 0x80000000
WS_CHILD        := 0x40000000
WS_MINIMIZE     := 0x20000000
WS_VISIBLE      := 0x10000000
WS_DISABLED     := 0x08000000
WS_CLIPSIBLINGS := 0x04000000
WS_CLIPCHILDREN := 0x02000000
WS_MAXIMIZE     := 0x01000000
WS_BORDER       := 0x00800000
WS_DLGFRAME     := 0x00400000
WS_VSCROLL      := 0x00200000
WS_HSCROLL      := 0x00100000
WS_SYSMENU      := 0x00080000
WS_SIZEBOX      := 0x00040000
WS_MINIMIZEBOX  := 0x00020000
WS_MAXIMIZEBOX  := 0x00010000
WS_OVERLAPPED   := 0x00000000

/* コマンドオプション数チェック */
if( A_Args.Length < 2 ){
	DispUsage()
}

/* モードとウィンドウ指定 */
strPosSize := A_Args[1]
strParam := A_Args[2]
iAct := 0
iNoDisp := 0
iDebug := 0
iPost := 0
UniqueID := 0

;両端のダブルクォートを削除し、カンマ区切りを配列に分割
strPosSize := StrReplace(strPosSize, '"', "")
aryTemp := StrSplit(strPosSize, ",")
;座標とサイズを数値化
aryTemp[1] := Number(aryTemp[1])
aryTemp[2] := Number(aryTemp[2])
aryTemp[3] := Number(aryTemp[3])
aryTemp[4] := Number(aryTemp[4])

/* デバッグモードの指定 */
if( A_Args.Length >= 5 ){
	iDebug := 1
}

;MsgBox("SizePos: " String(aryTemp[1]) "," String(aryTemp[2]) "," String(aryTemp[3]) "," String(aryTemp[4]) )

/* フルパスに合致するウィンドウを列挙(非表示ウィンドウも対象としているため沢山ヒットする) */
listWin := WinGetList(strParam)
idxWin := 0
findCount := 0

/* ウィンドウスタイルでマスク */
for hWin in listWin{
	idxWin := idxWin + 1
	style := WinGetStyle("ahk_id " hWin)
	if( style & WS_VISIBLE and not(style & WS_DISABLED) and not(style & WS_POPUP) ){
		findCount := findCount + 1
		UniqueID := hWin
		if( iDebug ){
			MsgBox("Class[" idxWin "]=" WinGetClass("ahk_id " hWin) " / style=0x" StrUpper(Format("{1:X}",style)) )
		}
	}
}

/* エラーダイアログの扱い */
if( A_Args.Length >= 4 ){
	iNoDisp := A_Args[4]
}
if( StrUpper(iNoDisp) = "ON" or StrUpper(iNoDisp) = "NODISP" ){
	iNoDisp := 1
}

if( UniqueID = 0 ){
	if( (not(iNoDisp) or iDebug) ){
		MsgBox(strParam " not found.")
	}
	ExitApp()
}

if( findCount != 1 ){
	if( (not(iNoDisp) or iDebug) ){
		MsgBox(strParam " has multiple windows.")
	}
	ExitApp()
}

/* WinActivateを実施する場合 */
if( A_Args.Length >= 3 ){
	iAct := A_Args[3]
}
if( iAct = 1 or StrUpper(iAct) = "ON" or StrUpper(iAct) = "ACTIVATE" ){
	try{
		WinActivate("ahk_id " UniqueID)
	}catch TargetError as err{
		if( not(iNoDisp) or iDebug ){
			MsgBox(strParam " can not activate")
		}
		ExitApp()
	}
}

style := WinGetStyle("ahk_id " UniqueID)
;MsgBox("Class=" WinGetClass("ahk_id " UniqueID) " / style=0x" StrUpper(Format("{1:X}",style)) )

if( WinGetMinMax("ahk_id " UniqueID) != 0 ){
	WinRestore("ahk_id " UniqueID)
}

WinMove( aryTemp[1], aryTemp[2], aryTemp[3], aryTemp[4], "ahk_id " UniqueID )

ExitApp()

DispUsage(){
	msg := 'Usage:`n'
	msg := msg 'MoveResizeWindow.exe PosSize Param (Activate)`n`n'
	msg := msg '[PosSize]`n'
	msg := msg '    "X, Y, Width, Height"`n'
	msg := msg '[Param]`n'
	msg := msg '   "window-title`n'
	msg := msg '    ahk_exe full-path`n'
	msg := msg '    ahk_class class-name..."`n'
	msg := msg '[Activate]`n'
	msg := msg '    1/on/activate = with WinActivate`n'
	msg := msg '[NoDispError]`n'
	msg := msg '    1/on/nodisp = not display ErrorDialog`n'
	MsgBox(msg)
	ExitApp()
}


